CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  eeprom_pin = 28
  'sdcard_pin = 16

  file_info_serial = 1
  file_info_fileio = 2
  
  file_info_type   = 0
  file_info_data   = 1
  file_info_size   = 8

OBJ
  rendez : "rendezvous"
  fileio : "cfileio0"
  mem : "cmalloc"
  ser : "cserial"
  serp : "cserialp"
  'i2c_p : "pasm_i2c_driver_p"
  spi_p : "safe_spi_p"

DAT
  sdcard_pin long 100

PUB main | i, blocklongs, num, index, outfile, infile, argc, startaddr, endaddr, TopAddr, TopObjList
  longfill($7f00, 0, 256)
  start1(31, 30, %10000, 57600, 200)
  AddToCogTable(0, 1, string("bootsd.binary"), 0, long[rendez#memfreelist])
  AddToCogTable(1, 2, string("serial I/O"), 0, 0)
  'num := i2c_p.Initialize(eeprom_pin)
  'AddToCogTable(num - 1, 2, string("I2C Driver"), 0, 0)
  'fileio.start
  repeat
    ser.dbprintf0(string("Press any key to start SPINIX\n"))
    if (ser.rxcheck <> -1)
      quit
    waitcnt(clkfreq+cnt)

  ser.dbprintf0(string("Enter SD DO pin number (negative for reverse order): "))
  sdcard_pin := 0
  index := 0
  repeat
    i := ser.rx
    ser.tx(i)
    if i == "-"
      index := 1
    elseif i < "0" OR i > "9"
      quit
    sdcard_pin := (sdcard_pin * 10) + i - "0"

  if index
    num := spi_p.start_explicit(sdcard_pin, sdcard_pin-1, sdcard_pin-2, sdcard_pin-3)
  else
    num := spi_p.start(sdcard_pin)
  AddToCogTable(num - 1, 2, string("SPI Driver"), 0, 0)

  StartFileIO

  if fileio.mount(1, string("disk1"), sdcard_pin) =< 0
    ser.dbprintf0(STRING("\nmount failed.\n"))
    'ser.dbprintf0(STRING("Run install to initialize the file system\n"))
    repeat
  fileio.chdir(string("/"))

  run(string("shell"))

  repeat

PUB AddToCogTable(cognum, type, name, startaddr, size) | entryptr
  entryptr := rendez#cogtable + (cognum * 24)
  byte[entryptr] := type
  bytemove(entryptr+1, name, 19)
  byte[entryptr+19] := 0
  word[entryptr+20] := startaddr
  word[entryptr+22] := size

PUB run(fname) | infile, size, AppAddr, i, cognum
  ser.dbprintf1(string("run %s\n"), fname)
  'fileio.CheckMallocSpace
  infile := fileio.fopen(fname, string("r"))
  ifnot infile
    ser.dbprintf1(string("Could not open %s\n"), fname)
    return
  ' Determine the file size
  size := fileio.getfilesize(infile)
  {
  fileio.fseek(infile, 0, c#SEEK_END)
  size := fileio.ftell(infile)
  fileio.fseek(infile, 0, c#SEEK_SET
  }
  'fileio.CheckMallocSpace
  ' Allocate the memory plus some for the stack
  AppAddr := mem.calloc(size + 800)
  ifnot AppAddr
    ser.dbprintf1(string("Not enough memory to run %s\n"), fname)
    ser.dbprintf1(string("size = %d\n"), size)
    fileio.fclose(infile)
    return
  'fileio.CheckMallocSpace
  'fileio.memset(AppAddr, $a5, size + 400)
  fileio.fread(AppAddr, 1, size, infile)
  fileio.fclose(infile)
  'fileio.CheckMallocSpace
  ' Fixup
  Repeat i from 0 to 4
    Word[AppAddr+6+i<<1] += AppAddr
  'fileio.memset(sp, $a5, 100)
  byte[AppAddr] := 1
  cognum := Cognew($F004, AppAddr + 4)
  AddToCogTable(cognum, 1, fname, AppAddr, size + 800)

  repeat while byte[AppAddr]

  ser.dbprintf1(string("Stopping cog %d\n"), cognum)
  cogstop(cognum)

  mem.free(AppAddr)

DAT
  fileio1 FILE "cfileiosd1.binary"

PUB StartFileIO | AppAddr, size, i, cognum
  ser.dbprintf0(string("StartFileIO\n"))
  size := word[@fileio1 + $a] ' Start of stack
  ' Allocate the memory plus some for the stack
  AppAddr := mem.calloc(size + 800)
  ifnot AppAddr
    ser.dbprintf0(string("Not enough memory to run file I/O\n"))
    return
  bytemove(AppAddr, @fileio1, size)
  ' Fixup
  Repeat i from 0 to 4
    Word[AppAddr+6+i<<1] += AppAddr
  cognum := Cognew($F004, AppAddr + 4)
  AddToCogTable(cognum, 1, string("cfileiosd1.binary"), AppAddr, size + 800)
  waitcnt(clkfreq/100+cnt)

PUB start1(rxpin, txpin, mode, baudrate, stacksize) | stdio, stdin, stdout
{{
  This routine calls the malloc initialization routine.  The number of longs in
  the top object stack is set by the value of "stacksize".  It also calls the
  openserial function to initialize the standard I/O serial port.  It returns the
  stdio pointer if successful, or NULL if not.
}}
  mem.mallocinit(stacksize)
  stdio := openserial(rxpin, txpin, mode, baudrate, 512, 64)
  stdin := mem.malloc(file_info_size)
  stdout := mem.malloc(file_info_size)
  long[rendez#stdio] := stdio
  long[rendez#stdin] := stdin
  long[rendez#stdout] := stdout
  bytemove(stdin, stdio, file_info_size)
  bytemove(stdout, stdio, file_info_size)
  return stdio

PUB openserial(rxpin, txpin, mode, baudrate, rxsize, txsize) | pfileinfo, file_data
{{
  This routine allocates memory for a file info and serial port data structure.
  It calls the start1 function in the cserial object to create a serial port in
  the next available cog.  It returns a pointer to the file info structure if
  successful, or a NULL value if not successful.
}}
  pfileinfo := mem.malloc(file_info_size + ser#header_size + rxsize + txsize)
  ifnot pfileinfo
    return 0
  file_data := pfileinfo + file_info_size
  'file_data := ser.gethandle1 
  long[pfileinfo][file_info_type] := file_info_serial
  long[pfileinfo][file_info_data] := file_data
  ifnot serp.start1(file_data, rxpin, txpin, mode, baudrate, rxsize, txsize)
    mem.free(pfileinfo)
    return 0
  return pfileinfo
