CON
  _clkmode = xtal1 + pll16x
  _xinfreq = 5_000_000

  eeprom_pin = 28

OBJ
  mem : "cmalloc"
  eeprom : "pasm_I2C_Driver"
  ser : "cserial"
  serp : "cserialp"
  i2c_p : "pasm_i2c_driver_p"
  fileio : "cfileio"

PUB main
  longfill($7f00, 0, 256)
  mem.mallocinit(200)
  serp.start(31, 30, %10000, 57600)
  i2c_p.Initialize(eeprom_pin)

  repeat
    ser.dbprintf0(string("Press any key to begin installation\n"))
    if (ser.rxcheck <> -1)
      quit
    waitcnt(clkfreq+cnt)

  install

DAT
  boot byte "boot.binary", 0
  file "boot.binary"
  shell byte "shell.binary", 0
  file "shell.binary"
  rb byte "rb", 0
  file "rb"
  help byte "help.txt", 0
  file "help.txt"
  last byte 0
  list long @boot, @shell, @rb, @help, @last

PUB install | fileptr, listptr, nextptr, len, outfile, dataptr, addr, value
  repeat
    ser.dbprintf0(string("Enter the EEPROM size (32K, 64K or 128K): "))
    result := 0
    repeat
      value := ser.rx
      ser.tx(value)
      if (value == 13)
        quit
      if value => "0" AND value =< "9"
        result := (result * 10) + value - "0"
    if result == 32 OR result == 64 OR result == 128
      quit
 
  ser.dbprintf1(string("Formatting a %dK EEPROM\n"), result)
  fileio.mount(1, string("/"), eeprom_pin)
  fileio.format(result << 10)

  ser.dbprintf0(string("Installing SPINIX\n"))

  listptr := @list
  fileptr := @@long[listptr]
  listptr += 4
  nextptr := @@long[listptr]
  dataptr := fileptr + strsize(fileptr) + 1
  len := nextptr - dataptr
  ser.dbprintf2(string("Writing %s, %d bytes\n"), fileptr, len)
  repeat addr from 0 to len - 1 step 64
    eeprom.WritePage(eeprom_pin, $a0, addr, dataptr, 64)
    repeat while eeprom.WriteWait(eeprom_pin, $a0, addr)
    dataptr += 64

  fileptr := nextptr

  repeat while byte[fileptr]
    listptr += 4
    nextptr := @@long[listptr]
    dataptr := fileptr + strsize(fileptr) + 1
    len := nextptr - dataptr
    ser.dbprintf2(string("Writing %s, %d bytes\n"), fileptr, len)
    outfile := fileio.fopen(fileptr, string("w"))
    ifnot outfile
      ser.dbprintf1(string("Could not open output file %s\n"), fileptr)
    else
      fileio.fwrite(dataptr, 1, len, outfile)
      fileio.fclose(outfile)
    ifnot strcomp(fileptr, string("help.txt"))
      fileio.chmod(fileptr, %0111)
    fileptr := nextptr
  ser.dbprintf0(string("Installation Complete.  Rebooting\n"))

  waitcnt(clkfreq + cnt)

  reboot
